/*
 * Predictive-ish dictionary thing....
 *
 * 2003, Petter Knudsen
 *
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>


#include "predictive_dictionary.h"

/*
 * Creates a new predictiv dictionary, with the paths to the dictionaries as its arguments. _common_dict_filename_ is the path 
 * to the statistical dictionary and _word_dict_filename_ is the path to the static dictionary it falls back on. The dictionary
 * must be of a format supported by the dictionary system.
 *
 * Returns the newly created dictionary.
 *
 */
PredictiveDictionary* predictive_dictionary_new( char* common_dict_filename, char* word_dict_filename ) {

    PredictiveDictionary* dict = (PredictiveDictionary*) malloc( sizeof( PredictiveDictionary ) );
    
    dict->word_dictionary = dictionary_new_from_file( DICTIONARY_NORMAL, word_dict_filename );
    dict->common_dictionary = dictionary_new_from_file( DICTIONARY_STATISTICAL, common_dict_filename );

    dict->cycle_dictionary = NULL;
    dict->last_predicted_word = NULL;
    dict->learning = 1;
    
    if( dict->common_dictionary )
        dict->common_dictionary_filename = strdup( common_dict_filename );
    else
        dict->common_dictionary_filename = NULL;

    return dict;

}

/* 
 * Updates the dictionary with the word _word_. 
 *
 * Returns nothing(void)
 *
 */
void predictive_dictionary_update_with_word( PredictiveDictionary* dict, char* word ) {

    if( dict->learning || dictionary_has_word( dict->common_dictionary, word ) )
        dictionary_add_word( dict->common_dictionary, word, TRUE );

    dictionary_predict_reset( dict->common_dictionary ); 
    dictionary_predict_reset( dict->word_dictionary ); 

}

/*
 * Destroys the dictionary _dict_ and frees the allocated resources. If the common dictionary exists, the dictionary is
 * saved along with its statistics.
 *
 * Returnos nothing(void).
 *
 */
void predictive_dictionary_destroy( PredictiveDictionary* dict ) {

    if( dict->word_dictionary )
        dictionary_destroy( dict->word_dictionary );

    if( dict->common_dictionary ) {

        if( dict->common_dictionary_filename )
            dictionary_save( dict->common_dictionary, dict->common_dictionary_filename ); 

        dictionary_destroy( dict->common_dictionary );

    }

    free( dict );
}


/*
 * Tries to predict a word starting with _word_. 
 *
 * Returns the predicted word, or NULL uppon failure.
 *
 */
char* predictive_dictionary_predict( PredictiveDictionary* dict, char* word ) {

    char* predicted_word;

    predicted_word = dictionary_predict_word( dict->common_dictionary, word, FALSE );

    if( predicted_word == NULL ) {

        predicted_word = dictionary_predict_word( dict->word_dictionary, word, FALSE );

        if( predicted_word )
            dict->cycle_dictionary = dict->word_dictionary;
        else
            dict->cycle_dictionary = NULL;

    }
    else
        dict->cycle_dictionary = dict->common_dictionary;

    dict->last_predicted_word = predicted_word;

    return predicted_word;

}


/*
 * Tries to predict a word starting with _word_, and cycles possible competions when called several times.
 *
 * Returns the predicted word, or NULL uppon failure.
 *
 */

char* predictive_dictionary_cycle( PredictiveDictionary* dict, char* word ) {


    char* predicted_word;
    Dictionary* start_dict;

    if( dict->cycle_dictionary == NULL ||  word == NULL || word[0] == '\0' )
        return NULL;
    
    predicted_word = dictionary_predict_word( dict->cycle_dictionary, word, TRUE );

    if( predicted_word == NULL || predicted_word == dict->last_predicted_word ) {


        start_dict = dict->cycle_dictionary;
        do {

            if( dict->cycle_dictionary == dict->common_dictionary )
                dict->cycle_dictionary = dict->word_dictionary;
            else 
                dict->cycle_dictionary = dict->common_dictionary;

            predicted_word = dictionary_predict_word( dict->cycle_dictionary, word, TRUE );


        } while( predicted_word == NULL && start_dict != dict->cycle_dictionary );


    }

    dict->last_predicted_word = predicted_word;
    return predicted_word;
  
}



void predictive_dictionary_set_learning( PredictiveDictionary* dict, int learn ) {

    dict->learning = learn;

}
